@@ -27,7 +27,7 @@ class FieldValidationError(ValidationError):
     Validation error that refers to a specific field and has `fieldname` and `value` attributes.
     """
     def __init__(self, message, fieldname, value, path=''):
-        message = "Value {!r} for field '{}' {}".format(value, path, message)
+        message = "Value {0!r} for field '{1}' {2}".format(value, path, message)
         super(FieldValidationError, self).__init__(message)
         self.fieldname = fieldname
         self.value = value
@@ -52,7 +52,7 @@ class RequiredFieldValidationError(ValidationError):
 
 class MultipleValidationError(ValidationError):
     def __init__(self, errors):
-        msg = "{} validation errors:\n{}".format(len(errors), '\n'.join(str(e) for e in errors))
+        msg = "{0} validation errors:\n{1}".format(len(errors), '\n'.join(str(e) for e in errors))
         super(MultipleValidationError, self).__init__(msg)
         self.errors = errors
 
@@ -60,10 +60,14 @@ class MultipleValidationError(ValidationError):
 def _generate_datetime_validator(format_option, dateformat_string):
     def validate_format_datetime(validator, fieldname, value, format_option):
         try:
-            # Additions to support date-time with microseconds without breaking existing date-time validation.
-            # Microseconds will appear specifically separated by a period, as some variable length decimal number
-            # such as  '2015-11-18T19:57:05.061Z'  instead of  '2015-11-18T19:57:05Z'  Better would be to use
-            # strict_rfc3339 vs datetime.strptime though the user needs the package installed for the import to succeed.
+            # Additions to support date-time with microseconds without breaking
+            # existing date-time validation.
+            # Microseconds will appear specifically separated by a period, as
+            # some variable length decimal number
+            # such as  '2015-11-18T19:57:05.061Z' instead of
+            # '2015-11-18T19:57:05Z'  Better would be to use
+            # strict_rfc3339 vs datetime.strptime though the user needs the
+            #  package installed for the import to succeed.
             #     import strict_rfc3339
             #     assert strict_rfc3339.validate_rfc3339(value)
             if format_option == 'date-time' and '.' in value:
@@ -157,6 +161,11 @@ class SchemaValidator(object):
     def register_format_validator(self, format_name, format_validator_fun):
         self._format_validators[format_name] = format_validator_fun
 
+    def get_default(self, value):
+        if isinstance(value, dict) or isinstance(value, list):
+            return copy.deepcopy(value)
+        return value
+
     def validate_type_string(self, val):
         return isinstance(val, _str_type)
 
@@ -198,24 +207,24 @@ class SchemaValidator(object):
         else:
             self._errors.append(err)
 
-    def _validate_unknown_properties(self, schema, data, fieldname):
+    def _validate_unknown_properties(self, schema, data, fieldname, patternProperties):
         """Raise a SchemaError when unknown fields are found."""
         schema_properties = set(schema)
         data_properties = set(data)
         delta = data_properties - schema_properties
-        if delta:
-            unknowns = ', '.join(['"{}"'.format(x) for x in delta])
+        if self.disallow_unknown_properties and delta:
+            unknowns = ', '.join(['"{0}"'.format(x) for x in delta])
             raise SchemaError('Unknown properties for field "{fieldname}": {unknowns}'.format(
                 fieldname=fieldname, unknowns=unknowns))
 
-    def _remove_unknown_properties(self, schema, data, fieldname):
-        """Remove the unknown fields from the data."""
-        schema_properties = set(schema)
-        data_properties = set(data)
-        unknown_fields = data_properties - schema_properties
+        elif self.remove_unknown_properties:
+            patterns = patternProperties.keys() if patternProperties else []
 
-        for unknown_field in unknown_fields:
-            del data[unknown_field]
+            if patterns:
+                delta = [f for f in delta if not any(re.match(p, f) for p in patterns)]
+
+            for unknown_field in delta:
+                del data[unknown_field]
 
     def validate_type(self, x, fieldname, schema, path, fieldtype=None):
         ''' Validates that the fieldtype specified is correct for the given data '''
@@ -240,13 +249,13 @@ class SchemaValidator(object):
                         errors = self._errors[:]
                         self.validate_type(x, fieldname, eachtype, path, eachtype)
                         if len(self._errors) > len(errors):
-                          # an exception was raised.
-                          # remove the error from self.errors and raise it here
-                          raise self._errors.pop()
+                            # an exception was raised.
+                            # remove the error from self.errors and raise it here
+                            raise self._errors.pop()
 
                         datavalid = True
                         break
-                    except ValidationError as err:
+                    except (SchemaError, ValidationError) as err:
                         errorlist.append(err)
                 if not datavalid:
                     self._error("doesn't match any of {numsubtypes} subtypes in {fieldtype}; "
@@ -262,7 +271,7 @@ class SchemaValidator(object):
                 try:
                     type_checker = getattr(self, 'validate_type_' + fieldtype)
                 except AttributeError:
-                    raise SchemaError("Field type '{}' is not supported.".format(fieldtype))
+                    raise SchemaError("Field type '{0}' is not supported.".format(fieldtype))
 
                 if not type_checker(value):
                     self._error("is not of type {fieldtype}", value, fieldname, path=path,
@@ -275,16 +284,15 @@ class SchemaValidator(object):
             if isinstance(value, dict):
                 if isinstance(properties, dict):
 
-                    if self.disallow_unknown_properties:
-                        self._validate_unknown_properties(properties, value, fieldname)
-                    elif self.remove_unknown_properties:
-                        self._remove_unknown_properties(properties, value, fieldname)
+                    if self.disallow_unknown_properties or self.remove_unknown_properties:
+                        self._validate_unknown_properties(properties, value, fieldname,
+                                                          schema.get('patternProperties'))
 
                     for property in properties:
                         self.__validate(property, value, properties.get(property),
                                         path + '.' + property)
                 else:
-                    raise SchemaError("Properties definition of field '{}' is not an object"
+                    raise SchemaError("Properties definition of field '{0}' is not an object"
                                       .format(fieldname))
 
     def validate_items(self, x, fieldname, schema, path, items=None):
@@ -302,23 +310,23 @@ class SchemaValidator(object):
                         for index, item in enumerate(items):
                             try:
                                 self.__validate("_data", {"_data": value[index]}, item,
-                                                '{}[{}]'.format(path, index))
+                                                '{0}[{1}]'.format(path, index))
                             except FieldValidationError as e:
                                 raise type(e)("Failed to validate field '%s' list schema: %s" %
                                               (fieldname, e), fieldname, e.value)
                 elif isinstance(items, dict):
                     for index, item in enumerate(value):
-                        if self.disallow_unknown_properties and 'properties' in items:
-                            self._validate_unknown_properties(items['properties'], item,
-                                                              fieldname)
-                        elif self.remove_unknown_properties and 'properties' in items:
-                            self._remove_unknown_properties(items['properties'], item,
-                                                            fieldname)
+                        if ((self.disallow_unknown_properties or
+                             self.remove_unknown_properties) and 'properties' in items):
+                            self._validate_unknown_properties(items['properties'],
+                                                              item,
+                                                              fieldname,
+                                                              schema.get('patternProperties'))
 
                         self.__validate("[list item]", {"[list item]": item}, items,
-                                        '{}[{}]'.format(path, index))
+                                        '{0}[{1}]'.format(path, index))
                 else:
-                    raise SchemaError("Properties definition of field '{}' is "
+                    raise SchemaError("Properties definition of field '{0}' is "
                                       "not a list or an object".format(fieldname))
 
     def validate_required(self, x, fieldname, schema, path, required):
@@ -398,7 +406,7 @@ class SchemaValidator(object):
                     self.__validate(eachProperty, value, additionalProperties, path)
         else:
             raise SchemaError("additionalProperties schema definition for "
-                              "field '{}' is not an object".format(fieldname))
+                              "field '{0}' is not an object".format(fieldname))
 
     def validate_dependencies(self, x, fieldname, schema, path, dependencies=None):
         if x.get(fieldname) is not None:
@@ -453,6 +461,26 @@ class SchemaValidator(object):
                     self._error("is greater than maximum value: {maximum}", value, fieldname,
                                 maximum=maximum, path=path)
 
+    def validate_maxProperties(self, x, fieldname, schema, path, number=None):
+        '''
+        Validates that the number of properties of the given object is less than or equal
+        to the specified number
+        '''
+        value = x.get(fieldname)
+        if isinstance(value, dict) and len(value) > number:
+            self._error("must have number of properties less than or equal to {number}",
+                        value, fieldname, number=number, path=path)
+
+    def validate_minProperties(self, x, fieldname, schema, path, number=None):
+        '''
+        Validates that the number of properties of the given object is greater than or equal
+        to the specified number
+        '''
+        value = x.get(fieldname)
+        if isinstance(value, dict) and len(value) < number:
+            self._error("must have number of properties greater than or equal to {number}",
+                        value, fieldname, number=number, path=path)
+
     def validate_maxLength(self, x, fieldname, schema, path, length=None):
         '''
         Validates that the value of the given field is shorter than or equal
@@ -499,9 +527,9 @@ class SchemaValidator(object):
         Validates that the given field, if a string, matches the given regular expression.
         '''
         value = x.get(fieldname)
-        if (isinstance(value, _str_type) and (isinstance(pattern, _str_type) and not
-                re.match(pattern, value) or not isinstance(pattern, _str_type) and not
-                pattern.match(value))):
+        if (isinstance(value, _str_type) and
+            (isinstance(pattern, _str_type) and not re.match(pattern, value)
+             or not isinstance(pattern, _str_type) and not pattern.match(value))):
             self._error("does not match regular expression '{pattern}'", value, fieldname,
                         pattern=pattern, path=path)
 
@@ -544,7 +572,7 @@ class SchemaValidator(object):
             if callable(options):
                 options = options(x)
             if not isinstance(options, Container):
-                raise SchemaError("Enumeration {!r} for field '{}' must be a container".format(
+                raise SchemaError("Enumeration {0!r} for field '{1}' must be a container".format(
                                   options, fieldname))
             if value not in options:
                 if not(value == '' and schema.get('blank', self.blank_by_default)):
@@ -553,11 +581,11 @@ class SchemaValidator(object):
 
     def validate_title(self, x, fieldname, schema, path, title=None):
         if not isinstance(title, (_str_type, type(None))):
-            raise SchemaError("The title for field '{}' must be a string".format(fieldname))
+            raise SchemaError("The title for field '{0}' must be a string".format(fieldname))
 
     def validate_description(self, x, fieldname, schema, path, description=None):
         if not isinstance(description, (_str_type, type(None))):
-            raise SchemaError("The description for field '{}' must be a string".format(fieldname))
+            raise SchemaError("The description for field '{0}' must be a string".format(fieldname))
 
     def validate_divisibleBy(self, x, fieldname, schema, path, divisibleBy=None):
         value = x.get(fieldname)
@@ -566,7 +594,7 @@ class SchemaValidator(object):
             return
 
         if divisibleBy == 0:
-            raise SchemaError("'{!r}' <- divisibleBy can not be 0".format(schema))
+            raise SchemaError("'{0!r}' <- divisibleBy can not be 0".format(schema))
 
         if value % divisibleBy != 0:
             self._error("is not divisible by '{divisibleBy}'.", x.get(fieldname), fieldname,
@@ -614,7 +642,7 @@ class SchemaValidator(object):
             # add default values first before checking for required fields
             if self.apply_default_to_data and 'default' in schema:
                 try:
-                    self.validate_type(x={'_ds': schema['default']}, fieldname='_ds',
+                    self.validate_type(x={'_ds': self.get_default(schema['default'])}, fieldname='_ds',
                                        schema=schema,
                                        fieldtype=schema['type'] if 'type' in schema else None,
                                        path=path)
@@ -622,7 +650,7 @@ class SchemaValidator(object):
                     raise SchemaError(exc)
 
                 if fieldname not in data:
-                    data[fieldname] = schema['default']
+                    data[fieldname] = self.get_default(schema['default'])
 
             # iterate over schema and call all validators
             for schemaprop in newschema:
